@@ -4,31 +4,23 @@
 #' @param x Vector of taxa names (character) or IDs (character or numeric)
 #' to query. For `db = "eol"`, EOL expects you to pass it a taxon id, called
 #' `eolid` in the output of [get_eolid()]. 
-#' @param db character; database to query. either `ncbi`, `itis`, `eol`, `col`,
+#' @param db character; database to query. either `ncbi`, `itis`, `eol`,
 #' `tropicos`, `gbif`, `nbn`, `worms`, `natserv`, `bold`, `wiki`, or `pow`.
 #' Note that each taxonomic data source has, their own identifiers, so that
 #' if you provide the wrong `db` value for the identifier you could get a
 #' result, but it will likely be wrong (not what you were expecting). If using
-#' ncbi, eol, and/or tropicos, we recommend getting an API key; see
+#' ncbi, and/or tropicos, we recommend getting an API key; see
 #' [taxize-authentication]
 #' @param id character; identifiers, returned by [get_tsn()], [get_uid()],
-#' [get_eolid()], [get_colid()], [get_tpsid()], [get_gbifid()], [get_tolid()],
+#' [get_eolid()], [get_tpsid()], [get_gbifid()], [get_tolid()],
 #' [get_wormsid()], [get_natservid()], [get_wormsid()], [get_wiki()],
 #' [get_pow()]
 #' @param callopts Curl options passed on to [crul::verb-GET]
 #' @param ... For `classification`: other arguments passed to [get_tsn()],
-#' [get_uid()], [get_eolid()], [get_colid()], [get_tpsid()], [get_gbifid()],
+#' [get_uid()], [get_eolid()], [get_tpsid()], [get_gbifid()],
 #' [get_wormsid()], [get_natservid()], [get_wormsid()], [get_wiki()],
 #' [get_pow()]. For `rbind.classification` and `cbind.classification`: one or
 #' more objects of class `classification`
-#'
-#' @param start The first record to return. If omitted, the results are
-#' returned from the first record (start=0). This is useful if the total
-#' number of results is larger than the maximum number of results returned by
-#' a single Web service query (currently the maximum number of results returned
-#' by a single query is 500 for terse queries and 50 for full queries).
-#' @param checklist character; The year of the checklist to query, if you want
-#' a specific year's checklist instead of the lastest as default (numeric).
 #' @param return_id (logical) If `TRUE` (default), return the taxon id
 #' as well as the name and rank of taxa in the lineage returned.
 #' Ignored for natserv as they don't return IDs in their taxonomic
@@ -36,6 +28,10 @@
 #' @param rows (numeric) Any number from 1 to infinity. If the default NA,
 #' all rows are considered. Note that this parameter is ignored if you pass
 #' in a taxonomic id instead of a name of class character.
+#' @param batch_size (numeric) For NCBI queries, specify the number of IDs to
+#'   lookup for each query.
+#' @param max_tries (numeric) For NCBI queries, the number of times a particular
+#'   query will be attempted, assuming the first does not work.
 #'
 #' @return A named list of data.frames with the taxonomic classification of
 #'    every supplied taxa.
@@ -47,7 +43,7 @@
 #' classification. But you can attach it yourself quite easily of course.
 #' This behavior is different from the other data sources.
 #'
-#' @seealso [get_tsn()], [get_uid()], [get_eolid()], [get_colid()],
+#' @seealso [get_tsn()], [get_uid()], [get_eolid()],
 #'    [get_tpsid()], [get_gbifid()], [get_wormsid()], [get_natservid()],
 #'    [get_boldid()], [get_wiki()], [get_pow()]
 #'
@@ -116,20 +112,17 @@
 #' classification(c("Chironomus riparius", "aaa vva"), db = 'itis',
 #'   messages=FALSE)
 #' classification(c("Chironomus riparius", "aaa vva"), db = 'eol')
-#' classification(c("Chironomus riparius", "aaa vva"), db = 'col')
 #' classification("Alopias vulpinus", db = 'nbn')
 #' classification('Gadus morhua', db = 'worms')
 #' classification('Aquila chrysaetos', db = 'natserv')
 #' classification('Gadus morhua', db = 'natserv')
 #' classification('Pomatomus saltatrix', db = 'natserv')
 #' classification('Aquila chrysaetos', db = 'natserv')
-#' classification(c("Chironomus riparius", "aaa vva"), db = 'col',
-#'   messages=FALSE)
 #' classification(c("Chironomus riparius", "asdfasdfsfdfsd"), db = 'gbif')
 #' classification("Chironomus", db = 'tol')
 #' classification("Poa annua", db = 'tropicos')
 #'
-#' # Use methods for get_uid, get_tsn, get_eolid, get_colid, get_tpsid
+#' # Use methods for get_uid, get_tsn, get_eolid, get_tpsid
 #' classification(get_uid(c("Chironomus riparius", "Puma concolor")))
 #'
 #' classification(get_uid(c("Chironomus riparius", "aaa vva")))
@@ -137,7 +130,6 @@
 #' classification(get_tsn(c("Chironomus riparius", "aaa vva"),
 #'   messages = FALSE))
 #' classification(get_eolid(c("Chironomus riparius", "aaa vva")))
-#' classification(get_colid(c("Chironomus riparius", "aaa vva")))
 #' classification(get_tpsid(c("Poa annua", "aaa vva")))
 #' classification(get_gbifid(c("Poa annua", "Bison bison")))
 #'
@@ -153,7 +145,7 @@
 #'
 #' # Many names to get_ids
 #' (out <- get_ids(names=c("Puma concolor","Accipiter striatus"),
-#'   db = c('ncbi','itis','col')))
+#'   db = c('ncbi','itis')))
 #' (cl <- classification(out))
 #' rbind(cl)
 #' ## cbind with so many names results in some messy data
@@ -161,13 +153,6 @@
 #' ## so you can turn off return_id
 #' cbind( classification(out, return_id=FALSE) )
 #'
-#' # rbind and cbind on class classification (from a
-#' # call to get_colid, get_tsn, etc. other than get_ids)
-#' (cl_col <- classification(
-#'   get_colid(c("Puma concolor","Accipiter striatus"))))
-#' rbind(cl_col)
-#' cbind(cl_col)
-#'
 #' (cl_uid <- classification(get_uid(c("Puma concolor",
 #'   "Accipiter striatus")), return_id=FALSE))
 #' rbind(cl_uid)
@@ -204,6 +189,25 @@
 #' classification('Poa annua', db = 'tropicos', rows=1:4)
 #' classification('Poa annua', db = 'tropicos', rows=1)
 #' classification('Poa annua', db = 'tropicos', rows=6)
+#' 
+#' # Queries of many IDs are processed in batches for NCBI
+#' ids <- c("13083", "2650392", "1547764", "230054", "353934", "656984", 
+#' "271789", "126272", "184644", "73213", "662816", "1161803", "1239353", 
+#' "59420", "665675", "866969", "1091219", "1431218", "1471898", 
+#' "864321", "251768", "2486276", "2068772", "1825808", "2006532", 
+#' "128287", "1195738", "1084683", "1886461", "508296", "377247", 
+#' "1489665", "329325", "219243", "1176946", "339893", "197933", 
+#' "174510", "1704048", "212897", "154842", "1239280", "260135", 
+#' "405735", "1566412", "2083462", "651348", "983204", "165380", 
+#' "2338856", "2068760", "167262", "34229", "1213340", "478939", 
+#' "1933585", "49951", "1277794", "1671089", "1502538", "362355", 
+#' "746473", "242879", "158219", "313664", "2093188", "1541232", 
+#' "584742", "1331091", "147639", "284492", "75642", "1412882", 
+#' "391782", "1406855", "434506", "2053357", "217315", "1444328", 
+#' "329249", "2294004", "84942", "324458", "538247", "69452", "49170", 
+#' "1993845", "261646", "127633", "228146", "1420004", "1629772", 
+#' "577055", "697062", "231660", "648380", "554953", "746496", "2602969")
+#' result <- classification(ids, db = 'ncbi')
 #' }
 #'
 #' @examples \dontrun{
@@ -226,20 +230,15 @@ classification.default <- function(x, db = NULL, callopts = list(),
       stats::setNames(classification(id, return_id = return_id, ...), x)
     },
     ncbi = {
-      id <- process_ids(x, db, get_uid, rows = rows, ...)
+      id <- process_ids(x, db, get_uid, rows = rows)
       stats::setNames(classification(id, callopts = callopts,
-        return_id = return_id, ...), x)
+        return_id = return_id), x)
     },
     eol = {
       id <- process_ids(x, db, get_eolid, rows = rows, ...)
       stats::setNames(classification(id, callopts = callopts,
         return_id = return_id, ...), x)
     },
-    col = {
-      id <- process_ids(x, db, get_colid, rows = rows, ...)
-      stats::setNames(classification(id, callopts = callopts,
-        return_id = return_id, ...), x)
-    },
     tropicos = {
       id <- process_ids(x, db, get_tpsid, rows = rows, ...)
       stats::setNames(classification(id, callopts = callopts,
@@ -300,7 +299,6 @@ process_ids <- function(input, db, fxn, ...){
            itis = as.tsn,
            ncbi = as.uid,
            eol = as.eolid,
-           col = as.colid,
            tropicos = as.tpsid,
            gbif = as.gbifid,
            nbn = as.nbnid,
@@ -345,49 +343,84 @@ classification.tsn <- function(id, return_id = TRUE, ...) {
 
 #' @export
 #' @rdname classification
-classification.uid <- function(id, callopts = list(), return_id = TRUE, ...) {
+classification.uid <- function(id, callopts = list(), return_id = TRUE,
+  batch_size = 50, max_tries = 3, ...) {
+
   warn_db(list(...), "ncbi")
-  fun <- function(x, callopts){
+  fun <- function(x, callopts) {
     key <- getkey(NULL, service="entrez")
-    # return NA if NA is supplied
-    if (is.na(x)) {
-      out <- NA
-    } else {
-      query <- tc(list(db = "taxonomy", ID = x, api_key = key))
+    
+    query_ncbi <- function(ids) {
+      query <- tc(list(db = "taxonomy", ID = paste0(ids, collapse = ','),
+        api_key = key))
       cli <- crul::HttpClient$new(url = ncbi_base(),
         opts = c(http_version = 2L, callopts))
-      res <- cli$get("entrez/eutils/efetch.fcgi", query = query)
-      res$raise_for_status()
-      tt <- res$parse("UTF-8")
-      ttp <- xml2::read_xml(tt)
-      out <- data.frame(
-        name = xml_text_all(ttp,
-          "//TaxaSet/Taxon/LineageEx/Taxon/ScientificName"),
-        rank = xml_text_all(ttp, "//TaxaSet/Taxon/LineageEx/Taxon/Rank"),
-        id = xml_text_all(ttp, "//TaxaSet/Taxon/LineageEx/Taxon/TaxId"),
-        stringsAsFactors = FALSE)
-      parent_id <- xml_text_all(ttp, "//TaxaSet/Taxon/ParentTaxId") %||% ""
-      # Is not directly below root and no lineage info
-      if (NROW(out) == 0 && parent_id != "1") {
-        out <- NA
-      } else {
-        out <- rbind(out,
-          data.frame(
-            name = xml_text_all(ttp, "//TaxaSet/Taxon/ScientificName"),
-            rank = xml_text_all(ttp, "//TaxaSet/Taxon/Rank"),
-            id = xml_text_all(ttp, "//TaxaSet/Taxon/TaxId"),
-            stringsAsFactors = FALSE))
-        # Optionally return tsn of lineage
-        if (!return_id) out <- out[, c('name', 'rank')]
-        out$rank <- tolower(out$rank)
-        return(out)
+      success <- FALSE
+      tries <- 1
+      while (success == FALSE && tries <= max_tries) {
+        res <- cli$get("entrez/eutils/efetch.fcgi", query = query)
+        res$raise_for_status()
+        tt <- res$parse("UTF-8")
+        ttp <- xml2::read_xml(tt)
+        out <- lapply(xml2::xml_find_all(ttp, '//TaxaSet/Taxon'),
+          function(tax_node) {
+          lin <- data.frame(
+            name = xml_text_all(tax_node,
+              ".//LineageEx/Taxon/ScientificName"),
+            rank = xml_text_all(tax_node, ".//LineageEx/Taxon/Rank"),
+            id = xml_text_all(tax_node, ".//LineageEx/Taxon/TaxId"),
+            stringsAsFactors = FALSE)
+          targ_tax <- data.frame(
+            name = xml_text_all(tax_node, "./ScientificName"),
+            rank = xml_text_all(tax_node, "./Rank"),
+            id = xml_text_all(tax_node, "./TaxId"),
+            stringsAsFactors = FALSE)
+          rbind(lin, targ_tax)
+        })
+        # Is not directly below root and no lineage info
+        parent_id <- xml_text_all(ttp, "//TaxaSet/Taxon/ParentTaxId") %||% ""
+        out[vapply(out, NROW, numeric(1)) == 0 & parent_id != "1"] <- NA
+        # Add NA where the taxon ID was not found
+        names(out) <- xml_text(xml2::xml_find_all(ttp,
+          '//TaxaSet/Taxon/TaxId'))
+        out <- unname(out[ids])
+        success <- ! grepl(tt, pattern = 'error', ignore.case = TRUE)
+        tries <- tries + 1
+        # NCBI limits requests to three per second without key or 10 per
+        # second with key
+        ncbi_rate_limit_pause(key)
+        # Wait longer if query failed
+        if (success == FALSE) {
+          Sys.sleep(1)
+        }
       }
+      # Return NA if cannot get information
+      if (!success) {
+        out <- rep(list(NA), length(ids))
+        warning(call. = FALSE, 'Giving up on query after ',
+          max_tries, ' tries. NAs will be returned.')
+      }
+      return(out)
     }
-    # NCBI limits requests to three per second
-    if (is.null(key)) Sys.sleep(0.34)
+    # return NA if NA is supplied
+    out <- rep(list(NA), length(x))
+    out[! is.na(x)] <- query_ncbi(x[! is.na(x)])
+    # Optionally return taxon id of lineage taxa
+    if (!return_id) {
+      out[! is.na(out)] <- lapply(out[! is.na(out)],
+        function(o) o[, c('name', 'rank')])
+    }
+    # Return ranks in all lower case
+    out[! is.na(out)] <- lapply(out[! is.na(out)], function(o) {
+      o$rank <- tolower(o$rank)
+      return(o)
+    })
     return(out)
   }
-  out <- lapply(id, fun, callopts = callopts)
+  id <- as.character(id) # force to character
+  id_chunks <- split(id, ceiling(seq_along(id)/batch_size))
+  out <- lapply(id_chunks, fun, callopts = callopts)
+  out <- unlist(out, recursive = FALSE)
   names(out) <- id
   structure(out, class = 'classification', db = 'ncbi')
 }
@@ -401,7 +434,6 @@ classification.eolid <- function(id, callopts = list(), return_id = TRUE, ...) {
     if (is.na(x)) {
       out <- NA
     } else {
-      key <- getkey(NULL, "EOL_KEY")
       args <- tc(list(common_names = common_names, synonyms = synonyms))
       cli <- crul::HttpClient$new(url = 'https://eol.org', opts = callopts)
       tt <- cli$get(file.path('api/hierarchy_entries/1.0', paste0(x, ".json")),
@@ -415,7 +447,7 @@ classification.eolid <- function(id, callopts = list(), return_id = TRUE, ...) {
           z[sapply(z, is.null)] <- NA_character_
           data.frame(z, stringsAsFactors = FALSE)
         })
-        out <- do.call(rbind.fill, fff)[,c('scientificName',
+        out <- dt2df(fff, idcol = FALSE)[,c('scientificName',
           'taxonRank', 'taxonID')]
         # add querried taxon
         tr <- res$taxonRank
@@ -434,49 +466,6 @@ classification.eolid <- function(id, callopts = list(), return_id = TRUE, ...) {
   structure(out, class = 'classification', db = 'eol')
 }
 
-#' @export
-#' @rdname classification
-classification.colid <- function(id, start = NULL, checklist = NULL,
-                                 callopts = list(), return_id = TRUE, ...) {
-  warn_db(list(...), "col")
-  fun <- function(x, checklist, start, callopts){
-    # return NA if NA is supplied
-    if (is.na(x)) {
-      out <- NA
-    } else {
-      url <- make_url(checklist)
-      args <- tc(list(id = x, response = "full", start = start))
-      res <- tax_GET(url, query = args, opts = callopts)
-      out <- search_col_classification_df(res)
-      # add query-ied taxon
-      out <- rbind(out, c(
-        xml2::xml_text(xml2::xml_find_first(res, "//result/name")),
-        xml2::xml_text(xml2::xml_find_first(res, "//result/rank")),
-        xml2::xml_text(xml2::xml_find_first(res, "//result/id"))))
-      # Optionally return id of lineage
-      if (!return_id) out <- out[, c('name', 'rank')]
-      out$rank <- tolower(out$rank)
-    }
-    return(out)
-  }
-  out <- lapply(id, fun, checklist = checklist, start = start,
-    callopts = callopts)
-  names(out) <- id
-  structure(out, class = 'classification', db = 'col')
-}
-
-search_col_classification_df <- function(x) {
-  name <- xml2::xml_text(xml2::xml_find_all(x, "//classification//name"))
-  rank <- xml2::xml_text(xml2::xml_find_all(x, "//classification//rank"))
-  id <- xml2::xml_text(xml2::xml_find_all(x, "//classification//id"))
-  if (any(grepl("species", rank, ignore.case = TRUE))) {
-    name[which(rank %in% "Species")] <- paste(name[which(rank %in% "Genus")],
-      name[which(rank %in% "Species")], collapse = " ")
-  }
-  data.frame(name, rank, id, stringsAsFactors = FALSE)
-}
-
-
 #' @export
 #' @rdname classification
 classification.tpsid <- function(id, callopts = list(), return_id = TRUE, ...) {
@@ -489,12 +478,10 @@ classification.tpsid <- function(id, callopts = list(), return_id = TRUE, ...) {
       key <- getkey(NULL, "TROPICOS_KEY")
       args <- tc(list(format = 'json', apikey = key))
       tt <- tax_GET(url, query = args, opts = callopts)
-      out <- jsonlite::fromJSON(tt$parse("UTF-8"), FALSE)
-      if (names(out[[1]])[[1]] == "Error") {
+      if ("Error" %in% names(tt)) {
         out <- data.frame(ScientificName = NA, Rank = NA)
       } else {
-        out <- do.call(rbind.fill,
-          lapply(out, data.frame))[,c('ScientificName','Rank', 'NameId')]
+        out <- tt[,c('ScientificName','Rank', 'NameId')]
       }
       names(out) <- c('name', 'rank', 'id')
       # Optionally return id of lineage
@@ -521,16 +508,17 @@ classification.gbifid <- function(id, callopts = list(),
       out <- suppressWarnings(tryCatch(
         gbif_name_usage(key = x, callopts = callopts),
         error = function(e) e))
-      if (is(out, "simpleError")) {
+      if (inherits(out, "simpleError")) {
         NA
       } else {
-        nms <- ldply(out[c('kingdom','phylum','class',
-          'order','family','genus','species')])
-        keys <- unname(unlist(out[paste0(c('kingdom','phylum','class',
-          'order','family','genus','species'), "Key")]))
+        cls = c('kingdom','phylum','class', 'order','family','genus','species')
+        nms <- nmdlst2df(out[names(out) %in% cls])
+        keys <- unname(unlist(out[names(out) %in% paste0(cls, "Key")]))
         df <- data.frame(name = nms$V1, rank = nms$.id, id = keys,
           stringsAsFactors = FALSE)
         df$rank <- tolower(df$rank)
+        # sort to make sure ranks are in correct descending order
+        df <- df[order(vapply(df$rank, which_rank, 1)), ]
         # Optionally return id of lineage
         if (!return_id) df[, c('name', 'rank')] else df
       }
@@ -796,7 +784,7 @@ cbind.classification <- function(...) {
   }
   input <- x <- c(...)
   input <- input[vapply(x, function(z) inherits(z, "data.frame"), logical(1))]
-  tmp <- do.call(rbind.fill, lapply(input, gethiernames))
+  tmp <- dt2df(lapply(input, gethiernames), idcol = FALSE)
   tmp$query <- names(x)
   tmp$db <- attr(x, "db")
   tmp
@@ -811,7 +799,7 @@ rbind.classification <- function(...) {
   for (i in seq_along(x)) {
     x[[i]]$query <- names(x[i])
   }
-  df <- do.call(rbind.fill, x)
+  df <- dt2df(x, idcol = FALSE)
   df$db <- db
   return( df )
 }
@@ -839,14 +827,13 @@ cbind.classification_ids <- function(...) {
       values
     }
   }
-  dat <- do.call(rbind.fill, lapply(input, function(h){
+  dat <- dt2df(lapply(input, function(h){
     tmp <- lapply(h, gethiernames)
-    tmp <- do.call(rbind.fill, tmp)
+    tmp <- dt2df(tmp, idcol = FALSE)
     tmp$query <- names(h)
     tmp$db <- attr(h, "db")
     tmp
-  })
-  )
+  }), idcol = FALSE)
   move_col(tt = dat, y = c('query','db'))
 }
 
@@ -877,6 +864,6 @@ rbind.classification_ids <- function(...) {
     get[[i]] <- tmp
   }
 
-  tt <- if (length(get) == 1) get[[1]] else do.call(rbind.fill, get)
+  tt <- if (length(get) == 1) get[[1]] else dt2df(get, idcol = FALSE)
   move_col(tt, c('query', 'db'))
 }
